#!/usr/bin/perl -w
#!perl
#!/usr/bin/perl

#############################################################################
# Copyright 2010                                                            #
# Purpose: To filter through files containing results from fitted           #
# Gaussian peaks to SEC chromatographs of proteins from ppSILAC.            #
# Output pseudo-chromatogram for each peak									#
#############################################################################

#############################################################################
#                   UBC Centre for High-Throughput Biology                  #
#                 Organelle Proteomics Group                                #
#                       Leonard Foster                                      #
#             E-mail: ljfoster@interchange.ubc.ca                           #
#      WWW: http://www.chibi.ubc.ca/faculty/foster/                         #
#                                                                           #
#                                                                           #
#    FILENAME:   peaks_filter.pl                                            #
#                                                                           #
# CREATED: LJF 03-24-2010                                                   #
# UPDATED: LJF 05-12-2010                                                   #
# UPDATED: LJF 10-25-2010                                                   #
# UPDATED: LJF 5-3-2011                                                     #
#                                                                           #
#                                                                           #
#############################################################################

# Usage - must be in the same directory as all the data files
# Usage: perl peaks_profiles.pl -i <Protein descriptions table>
use Getopt::Std;
my %options;
getopt ('i', \%options);

%accessions = %allPeaks = ();
@files = ();
$y = $time = $fileNum = $currHeight = $currCentre = $currWidth = $numCoeffs = $rSq = $cver = $rmse = 0;
$fit = "";


#lower & upper time range limits
$minTime = 0;
$maxTime = 50;

#resolution to iterate:
$iter = 0.1;

#Output from Matlab script names files only by a sequential number
#Will be helpful to re-attach accession numbers and descriptions to the fitted peaks
open (STDIN, $options{"i"}) || die "Can't open file with accession numbers and descriptions!\n";
while (<>) 
{
    if ($_ =~ m/(\d+)\t(.*)/)
	{
		$accessions{$1} = "$2";
	}
}
close STDIN;

#parse data in from Matlab files
#assumes all files in the current directory whose name conforms to the naming convention (#.txt) are to be used
#glob all files in the directory and parse all peaks into a single hash
#key will be in the form <filenumber_retentiontime>
@files = <*>;
foreach $file (@files) {
	if ($file =~ m/(\d+)\.txt/)
	{
		$fileNum = $1;		
		#now open the file and parse the peak info out
		open (STDIN, $file) || die "Can't open $file!\n";
		$coeffLine = <>;
		$line = <>;
		$rmse = 0;
		$rSq = 0;
		$cver = 0;
		if ($line =~ m/(\d+\.*\d*)/) {
			$rmse = $1;
		}
		$line = <>;
		if ($line =~ m/(\d+\.*\d*)/) {
			$rSq = $1;
		}
		$line = <>;
		if ($line =~ m/(\d+\.*\d*)/) {
			$cver = $1;
		}
		chomp($coeffLine);
		@coeffs = split(/ /,$coeffLine);
		$pos = -1;
		$numCoeffs = @coeffs/3;
		while ($pos < @coeffs - 1) {
			$pos += 1;
			$currHeight = $coeffs[$pos];
			$pos += 1;
			$currCentre = $coeffs[$pos];
			$pos += 1;
			$currWidth = $coeffs[$pos];
			$allPeaks{"$fileNum\t$rmse\t$rSq\t$cver\t$numCoeffs\t$currCentre\t$currHeight\t$currWidth"} = $accessions{$fileNum};
		}
		close STDIN;
	}
}

foreach $fit (keys %allPeaks) {
	($fileNum, $rmse, $rSq, $cver, $numCoeffs, $currCentre, $currHeight, $currWidth) = split(/\t/, $fit);
	$time = $minTime - $iter;
	while ($time < $maxTime) {
		$time += $iter;
		$y = $currHeight*(exp((-1*(($time-$currCentre)*($time-$currCentre)))/(2*($currWidth*$currWidth))));
		if ($y < 0.05) {
			$allPeaks{$fit} .= "\t0";
		} else {
			$allPeaks{$fit} .= "\t$y";			
		}
	}
}

open (STDOUT, ">", "results.txt") || die "Can't create output file!\n";
foreach $prot (keys %allPeaks) {
	print "$prot\t$allPeaks{$prot}\n";
}
close STDOUT;
